@@ -446,9 +446,8 @@ ntfs_inode *ntfs_pathname_to_inode(ntfs_volume *vol, ntfs_inode *parent,
 		}
 	}
 
-	unicode = ntfs_calloc(MAX_PATH);
 	ascii = strdup(pathname);
-	if (!unicode || !ascii) {
+	if (!ascii) {
 		ntfs_log_debug("Out of memory.\n");
 		err = ENOMEM;
 		goto close;
@@ -466,7 +465,7 @@ ntfs_inode *ntfs_pathname_to_inode(ntfs_volume *vol, ntfs_inode *parent,
 			q++;
 		}
 
-		len = ntfs_mbstoucs(p, &unicode, MAX_PATH);
+		len = ntfs_mbstoucs(p, &unicode);
 		if (len < 0) {
 			ntfs_log_debug("Couldn't convert name to Unicode: %s.\n", p);
 			err = errno;
@@ -499,6 +498,9 @@ ntfs_inode *ntfs_pathname_to_inode(ntfs_volume *vol, ntfs_inode *parent,
 			goto close;
 		}
 
+		free(unicode);
+		unicode = NULL;
+
 		p = q;
 		while (p && *p && *p == PATH_SEP)
 			p++;
@@ -1034,7 +1036,7 @@ err_out:
  */
 static ntfs_inode *__ntfs_create(ntfs_inode *dir_ni,
 		ntfschar *name, u8 name_len, dev_t type, dev_t dev,
-		ntfschar *target, u8 target_len)
+		ntfschar *target, int target_len)
 {
 	ntfs_inode *ni;
 	int rollback_data = 0, rollback_sd = 0;
@@ -1203,6 +1205,8 @@ static ntfs_inode *__ntfs_create(ntfs_inode *dir_ni,
 	fn->last_data_change_time = utc2ntfs(ni->last_data_change_time);
 	fn->last_mft_change_time = utc2ntfs(ni->last_mft_change_time);
 	fn->last_access_time = utc2ntfs(ni->last_access_time);
+	fn->data_size = cpu_to_sle64(ni->data_size);
+	fn->allocated_size = cpu_to_sle64(ni->allocated_size);
 	memcpy(fn->file_name, name, name_len * sizeof(ntfschar));
 	/* Add FILE_NAME attribute to inode. */
 	if (ntfs_attr_add(ni, AT_FILE_NAME, AT_UNNAMED, 0, (u8*)fn, fn_len)) {
@@ -1281,10 +1285,11 @@ ntfs_inode *ntfs_create_device(ntfs_inode *dir_ni, ntfschar *name, u8 name_len,
 }
 
 ntfs_inode *ntfs_create_symlink(ntfs_inode *dir_ni, ntfschar *name, u8 name_len,
-		ntfschar *target, u8 target_len)
+		ntfschar *target, int target_len)
 {
 	if (!target || !target_len) {
-		ntfs_log_error("Invalid arguments.\n");
+		ntfs_log_error("%s: Invalid argument (%p, %d)\n", __FUNCTION__,
+			       target, target_len);
 		return NULL;
 	}
 	return __ntfs_create(dir_ni, name, name_len, S_IFLNK, 0,
@@ -1309,7 +1314,7 @@ int ntfs_check_empty_dir(ntfs_inode *ni)
 	/* Non-empty directory? */
 	if ((na->data_size != sizeof(INDEX_ROOT) + sizeof(INDEX_ENTRY_HEADER))){
 		/* Both ENOTEMPTY and EEXIST are ok. We use the more common. */
-		errno = EEXIST;
+		errno = ENOTEMPTY;
 		ntfs_log_debug("Directory is not empty\n");
 		ret = -1;
 	}
@@ -1324,7 +1329,7 @@ static int ntfs_check_unlinkable_dir(ntfs_inode *ni, FILE_NAME_ATTR *fn)
 	int ret;
 	
 	ret = ntfs_check_empty_dir(ni);
-	if (!ret || errno != EEXIST)
+	if (!ret || errno != ENOTEMPTY)
 		return ret;
 	/* 
 	 * Directory is non-empty, so we can unlink only if there is more than
@@ -1332,7 +1337,7 @@ static int ntfs_check_unlinkable_dir(ntfs_inode *ni, FILE_NAME_ATTR *fn)
 	 */
 	if ((link_count == 1) || 
 	    (link_count == 2 && fn->file_name_type == FILE_NAME_DOS)) {
-		errno = EEXIST;
+		errno = ENOTEMPTY;
 		ntfs_log_debug("Non-empty directory without hard links\n");
 		goto no_hardlink;
 	}
