@@ -81,12 +81,12 @@ int ntfs_mft_records_read(const ntfs_volume *vol, const MFT_REF mref,
 	s64 br;
 	VCN m;
 
-	ntfs_log_trace("Entering for inode %lld\n", MREF(mref));
+	ntfs_log_trace("inode %llu\n", (unsigned long long)MREF(mref));
 	
 	if (!vol || !vol->mft_na || !b || count < 0) {
 		errno = EINVAL;
-		ntfs_log_perror("%s: b=%p  count=%lld  mft=%lld", __FUNCTION__,
-				b, count, MREF(mref));
+		ntfs_log_perror("%s: b=%p  count=%lld  mft=%llu", __FUNCTION__,
+			b, (long long)count, (unsigned long long)MREF(mref));
 		return -1;
 	}
 	m = MREF(mref);
@@ -209,6 +209,40 @@ int ntfs_mft_records_write(const ntfs_volume *vol, const MFT_REF mref,
 	return -1;
 }
 
+int ntfs_mft_record_check(const ntfs_volume *vol, const MFT_REF mref, 
+			  MFT_RECORD *m)
+{			  
+	ATTR_RECORD *a;
+	int ret = -1;
+	
+	if (!ntfs_is_file_record(m->magic)) {
+		ntfs_log_error("Record %llu has no FILE magic (0x%x)\n",
+			       (unsigned long long)MREF(mref), *(le32 *)m);
+		goto err_out;
+	}
+	
+	if (le32_to_cpu(m->bytes_allocated) != vol->mft_record_size) {
+		ntfs_log_error("Record %llu has corrupt allocation size "
+			       "(%u <> %u)\n", (unsigned long long)MREF(mref),
+			       vol->mft_record_size,
+			       le32_to_cpu(m->bytes_allocated));
+		goto err_out;
+	}
+	
+	a = (ATTR_RECORD *)((char *)m + le16_to_cpu(m->attrs_offset));
+	if (p2n(a) < p2n(m) || (char *)a > (char *)m + vol->mft_record_size) {
+		ntfs_log_error("Record %llu is corrupt\n",
+			       (unsigned long long)MREF(mref));
+		goto err_out;
+	}
+	
+	ret = 0;
+err_out:
+	if (ret)
+		errno = EIO;
+	return ret;
+}
+
 /**
  * ntfs_file_record_read - read a FILE record from the mft from disk
  * @vol:	volume to read from
@@ -244,14 +278,13 @@ int ntfs_file_record_read(const ntfs_volume *vol, const MFT_REF mref,
 		MFT_RECORD **mrec, ATTR_RECORD **attr)
 {
 	MFT_RECORD *m;
-	ATTR_RECORD *a;
-	int err;
 
 	if (!vol || !mrec) {
 		errno = EINVAL;
 		ntfs_log_perror("%s: mrec=%p", __FUNCTION__, mrec);
 		return -1;
 	}
+	
 	m = *mrec;
 	if (!m) {
 		m = ntfs_malloc(vol->mft_record_size);
@@ -259,32 +292,26 @@ int ntfs_file_record_read(const ntfs_volume *vol, const MFT_REF mref,
 			return -1;
 	}
 	if (ntfs_mft_record_read(vol, mref, m)) {
-		err = errno;
 		ntfs_log_perror("ntfs_mft_record_read failed");
 		goto err_out;
 	}
-	err = EIO;
-	if (!ntfs_is_file_record(m->magic)) {
-		ntfs_log_perror("Record %llu has no FILE magic", MREF(mref));
+	if (ntfs_mft_record_check(vol, mref, m))
 		goto err_out;
-	}
+	
 	if (MSEQNO(mref) && MSEQNO(mref) != le16_to_cpu(m->sequence_number)) {
-		ntfs_log_perror("Record %llu has wrong SeqNo", MREF(mref));
-		goto err_out;
-	}
-	a = (ATTR_RECORD*)((char*)m + le16_to_cpu(m->attrs_offset));
-	if (p2n(a) < p2n(m) || (char*)a > (char*)m + vol->mft_record_size) {
-		ntfs_log_perror("Record %llu is corrupt", MREF(mref));
+		ntfs_log_error("Record %llu has wrong SeqNo (%d <> %d)\n",
+			       (unsigned long long)MREF(mref), MSEQNO(mref),
+			       le16_to_cpu(m->sequence_number));
+		errno = EIO;
 		goto err_out;
 	}
 	*mrec = m;
 	if (attr)
-		*attr = a;
+		*attr = (ATTR_RECORD*)((char*)m + le16_to_cpu(m->attrs_offset));
 	return 0;
 err_out:
 	if (m != *mrec)
 		free(m);
-	errno = err;
 	return -1;
 }
 
@@ -797,6 +824,7 @@ static int ntfs_mft_bitmap_extend_initialized(ntfs_volume *vol)
 	ll = ntfs_attr_pwrite(mftbmp_na, old_initialized_size, 8, &ll);
 	if (ll == 8) {
 		ntfs_log_debug("Wrote eight initialized bytes to mft bitmap.\n");
+		vol->free_mft_records += (8 * 8); 
 		return 0;
 	}
 	ntfs_log_error("Failed to write to mft bitmap.\n");
@@ -1440,6 +1468,7 @@ found_free_rec:
 	/* Return the opened, allocated inode of the allocated mft record. */
 	ntfs_log_debug("Returning opened, allocated %sinode 0x%llx.\n",
 			base_ni ? "extent " : "", (long long)bit);
+	vol->free_mft_records--; 
 	return ni;
 
 undo_mftbmp_alloc:
@@ -1508,8 +1537,10 @@ int ntfs_mft_record_free(ntfs_volume *vol, ntfs_inode *ni)
 	}
 
 	/* Throw away the now freed inode. */
-	if (!ntfs_inode_close(ni))
+	if (!ntfs_inode_close(ni)) {
+		vol->free_mft_records++; 
 		return 0;
+	}
 	err = errno;
 
 	/* Rollback what we did... */
