@@ -321,7 +321,7 @@ static int ntfs_fuse_parse_path(const char *org_path, char **path,
 		*path = strsep(&stream_name_mbs, ":");
 		if (stream_name_mbs) {
 			*stream_name = NULL;
-			res = ntfs_mbstoucs(stream_name_mbs, stream_name, 0);
+			res = ntfs_mbstoucs(stream_name_mbs, stream_name);
 			if (res < 0)
 				return -errno;
 			return res;
@@ -404,10 +404,11 @@ static int ntfs_fuse_getattr(const char *org_path, struct stat *stbuf)
 			 * Check whether it's Interix symbolic link, block or
 			 * character device.
 			 */
-			if (na->data_size <= sizeof(INTX_FILE_TYPES) + sizeof(
-					ntfschar) * MAX_PATH && na->data_size >
-					sizeof(INTX_FILE_TYPES) &&
-					!stream_name_len) {
+			if (na->data_size <= sizeof(INTX_FILE_TYPES) + 
+			    sizeof(ntfschar) * PATH_MAX && 
+			    na->data_size > sizeof(INTX_FILE_TYPES) && 
+			    !stream_name_len) {
+				
 				INTX_FILE *intx_file;
 
 				intx_file = ntfs_malloc(na->data_size);
@@ -501,7 +502,7 @@ static int ntfs_fuse_readlink(const char *org_path, char *buf, size_t buf_size)
 		goto exit;
 	}
 	if (na->data_size > sizeof(INTX_FILE_TYPES) +
-			sizeof(ntfschar) * MAX_PATH) {
+			sizeof(ntfschar) * PATH_MAX) {
 		res = -ENAMETOOLONG;
 		goto exit;
 	}
@@ -827,7 +828,7 @@ static int ntfs_fuse_create(const char *org_path, dev_t type, dev_t dev,
 	/* Generate unicode filename. */
 	name = strrchr(path, '/');
 	name++;
-	uname_len = ntfs_mbstoucs(name, &uname, 0);
+	uname_len = ntfs_mbstoucs(name, &uname);
 	if (uname_len < 0) {
 		res = -errno;
 		goto exit;
@@ -847,7 +848,7 @@ static int ntfs_fuse_create(const char *org_path, dev_t type, dev_t dev,
 					dev);
 			break;
 		case S_IFLNK:
-			utarget_len = ntfs_mbstoucs(target, &utarget, 0);
+			utarget_len = ntfs_mbstoucs(target, &utarget);
 			if (utarget_len < 0) {
 				res = -errno;
 				goto exit;
@@ -985,7 +986,7 @@ static int ntfs_fuse_link(const char *old_path, const char *new_path)
 	/* Generate unicode filename. */
 	name = strrchr(path, '/');
 	name++;
-	uname_len = ntfs_mbstoucs(name, &uname, 0);
+	uname_len = ntfs_mbstoucs(name, &uname);
 	if (uname_len < 0) {
 		res = -errno;
 		goto exit;
@@ -1039,7 +1040,7 @@ static int ntfs_fuse_rm(const char *org_path)
 	/* Generate unicode filename. */
 	name = strrchr(path, '/');
 	name++;
-	uname_len = ntfs_mbstoucs(name, &uname, 0);
+	uname_len = ntfs_mbstoucs(name, &uname);
 	if (uname_len < 0) {
 		res = -errno;
 		goto exit;
@@ -1456,7 +1457,7 @@ static int ntfs_fuse_getxattr(const char *path, const char *name,
 	ni = ntfs_pathname_to_inode(vol, NULL, path);
 	if (!ni)
 		return -errno;
-	lename_len = ntfs_mbstoucs(name + nf_ns_xattr_preffix_len, &lename, 0);
+	lename_len = ntfs_mbstoucs(name + nf_ns_xattr_preffix_len, &lename);
 	if (lename_len == -1) {
 		res = -errno;
 		goto exit;
@@ -1504,7 +1505,7 @@ static int ntfs_fuse_setxattr(const char *path, const char *name,
 	ni = ntfs_pathname_to_inode(vol, NULL, path);
 	if (!ni)
 		return -errno;
-	lename_len = ntfs_mbstoucs(name + nf_ns_xattr_preffix_len, &lename, 0);
+	lename_len = ntfs_mbstoucs(name + nf_ns_xattr_preffix_len, &lename);
 	if (lename_len == -1) {
 		res = -errno;
 		goto exit;
@@ -1562,7 +1563,7 @@ static int ntfs_fuse_removexattr(const char *path, const char *name)
 	ni = ntfs_pathname_to_inode(vol, NULL, path);
 	if (!ni)
 		return -errno;
-	lename_len = ntfs_mbstoucs(name + nf_ns_xattr_preffix_len, &lename, 0);
+	lename_len = ntfs_mbstoucs(name + nf_ns_xattr_preffix_len, &lename);
 	if (lename_len == -1) {
 		res = -errno;
 		goto exit;
@@ -1905,7 +1906,7 @@ static char *parse_mount_options(const char *orig_opts)
 		goto err_exit;
 	else if (ctx->atime == ATIME_ENABLED && strappend(&ret, "atime,"))
 		goto err_exit;
-	else if (strappend(&ret, "noatime,"))
+	else if (ctx->atime == ATIME_DISABLED && strappend(&ret, "noatime,"))
 		goto err_exit;
 	
 	if (strappend(&ret, "fsname="))
@@ -2172,17 +2173,13 @@ static struct fuse *mount_fuse(char *parsed_options)
 	struct fuse *fh = NULL;
 	struct fuse_args args = FUSE_ARGS_INIT(0, NULL);
 	
-	/* Libfuse can't always find fusermount, so let's help it. */
-	if (setenv("PATH", ":/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin", 0))
-		ntfs_log_perror("WARNING: Failed to set $PATH\n");
-	
 	ctx->fc = try_fuse_mount(parsed_options);
 	if (!ctx->fc)
 		return NULL;
 	
 	if (fuse_opt_add_arg(&args, "") == -1)
 		goto err;
-	if (fuse_opt_add_arg(&args, "-ouse_ino,kernel_cache") == -1)
+	if (fuse_opt_add_arg(&args, "-ouse_ino,kernel_cache,attr_timeout=0") == -1)
 		goto err;
 	if (ctx->debug)
 		if (fuse_opt_add_arg(&args, "-odebug") == -1)
